(*
 Oukseh Lee, The LET Project, KAIST

 This source is almost copied from the Objective Caml compiler.
*)

(***********************************************************************)
(*                                                                     *)
(*                           Objective Caml                            *)
(*                                                                     *)
(*            Xavier Leroy, projet Cristal, INRIA Rocquencourt         *)
(*                                                                     *)
(*  Copyright 1996 Institut National de Recherche en Informatique et   *)
(*  en Automatique.  All rights reserved.  This file is distributed    *)
(*  under the terms of the Q Public License version 1.0.               *)
(*                                                                     *)
(***********************************************************************)

(* $Id: ntopmain.n,v 1.2 2001/03/21 06:44:06 cookcu Exp $ *)

structure Ntopmain = struct
open Lexing
open Format
open Config
open Misc
open Parsetree
open Types
open Typedtree
open Nprintval
open N2caml
open Clflags
open Ntoploop

(*
let parse_toplevel_phrase = ref Parse.toplevel_phrase
*)
(*********** poisson ***********)

structure S = String_ast.Ast

local
 val sloc = Location.rhs_loc
 val nloc = Location.none
in
 fun mktopexp l =
        S.Str(
          S.SimpleDec(
            S.ValDec([],
              [(S.VarPat(("it",nloc),nloc),S.SeqExp(l,nloc),nloc)],
              nloc),
            nloc),
          nloc)
end

fun protect r newval body =
  let val oldval = !r 
  in  (r := newval;
       let val res = body() 
       in  r := oldval;
           res
       end) handle x => (r := oldval; raise x)
  end

fun typingit td = if !Clflags.ntyping then let val absyn = Infer.infer td
                                       in  NEnv.move_b_to_base()
                                       end
 
fun parse_toplevel_phr lb =
  let val nast = Nparse.toplevel_parse lb
     val _ = case nast of 
                S.TopDef d => ignore(typingit d) 
              | S.TopExp(e,i) => ignore(typingit(mktopexp e))
              | _ => ()
      val nast' = (Nconvert.convert_toplevel nast) 
  in  trans_toplevel nast'
  end

val parse_toplevel_phrase = ref parse_toplevel_phr

(*******************************)

(*
let parse_use_file = ref Parse.use_file
*)

(*********** poisson ***********)

fun parse_usefile lb =
  let val nast = Nparse.implementation_parse lb
  in  ignore(typingit nast);
      [Ptop_def (trans_topdec (Nconvert.convert_topdec nast))]
  end

val parse_use_file = ref parse_usefile

(*******************************)

fun use_file ppf name = 
  let val filename = find_in_path !Config.load_path name
      val ic = open_in_bin filename
      val lb = Lexing.from_channel ic
    (* Skip initial #! line if any *)
      val buffer = String.create 2
      val _ = if input ic buffer 0 2 = 2 && buffer = "#!"
              then ignore(input_line ic)
              else seek_in ic 0
      val success =
            protect Location.input_name filename (fn () =>
             ((List.iter
                (fn ph => (
                   if !Clflags.dump_parsetree then Printast.top_phrase ppf ph;
                   if not (execute_phrase !use_print_results ppf ph) then raise Exit))
                (!parse_use_file lb);
              true)
              handle
                Exit => false
              | Sys.Break => fprintf ppf "Interrupted.@."; false
              | x => NTerrors.report_error ppf x; false)) 
  in  close_in ic;
      success
  end handle Not_found => fprintf ppf "Cannot find file %s.@." name; false

fun use_silently ppf name =
  protect use_print_results false (fn () => use_file ppf name)

fun dir_use ppf name = ignore(use_file ppf name)

val _ = Hashtbl.add directive_table "use" (Directive_string (dir_use std_formatter))


(* Toplevel initialization. Performed here instead of at the
   beginning of loop() so that user code linked in with ocamlmktop
   can call directives from Topdirs. *)

val _ = (
  Sys.interactive := true;
  Symtable.init_toplevel();
  Clflags.thread_safe := true;
  NTcompile.init_path())

fun load_ocamlinit ppf =
  if Sys.file_exists ".ocamlinit" then ignore(use_silently ppf ".ocamlinit")

(* The interactive loop *)

exception PPerror

fun loop ppf = (
  Nconfig.print_full_header();
  (* Add whatever -I options have been specified on the command line,
     but keep the directories that user code linked in with ocamlmktop
     may have added to load_path. *)
(*******************)
  load_path := "" :: (List.rev !Clflags.include_dirs @ !load_path);
  toplevel_env := NTcompile.initial_env();
  let val lb = Lexing.from_function Ntoploop.refill_lexbuf 
  in  Location.input_name := "";
      Location.input_lexbuf := Some lb;
      Sys.catch_break true;
      load_ocamlinit ppf;
      while true do (
        Ntoploopin.empty_lexbuf lb;
        Location.reset();
        first_line := true;
        NEnv.reset();
        let val phr = (!parse_toplevel_phrase lb) handle Exit => raise PPerror 
        in  if !Clflags.dump_parsetree then Printast.top_phrase ppf phr;
            ignore(execute_phrase true ppf phr)
        end) handle End_of_file => exit 0
                  | Sys.Break => fprintf ppf "Interrupted.@."
                  | PPerror => ()
                  | x => NTerrors.report_error ppf x
      end
  end)

(* Execute a script *)

fun run_script ppf name args =
  let fun find n = (
        if n >= Array.length args then invalid_arg "Toploop.run_script";
        if Ntoploopin.record args n = name then n else find (n+1))
      val pos = find 0 
      val len = Array.length args - pos
  in  if Array.length Sys.argv < len then invalid_arg "Toploop.run_script";
      Array.blit args pos Sys.argv 0 len;
      Obj.truncate (Obj.repr Sys.argv) len;
      NTcompile.init_path();
      toplevel_env := NTcompile.initial_env();
      use_silently ppf name
  end

val usage = "Usage: nml <options> [script-file]\noptions are:"

fun file_argument name =
  exit (if run_script Format.err_formatter name Sys.argv then 0 else 2)

fun main () = (
  Arg.parse [
     ("-I", Arg.String(fn dir => include_dirs := dir :: !include_dirs),
           "<dir>  Add <dir> to the list of include directories"),
     ("-labels", Arg.Clear classic, " Use commuting label mode"),
     ("-modern", Arg.Clear classic, " (deprecated) same as -labels"),
     ("-noassert", Arg.Set noassert, " Do not compile assertion checks"),
     ("-rectypes", Arg.Set recursive_types, " Allow arbitrary recursive types"),
     ("-unsafe", Arg.Set fast, " No bound checking on array and string access"),
     ("-w", Arg.String Warnings.parse_options,
           "<flags>  Enable or disable warnings according to <flags>:\n\
       \032    A/a enable/disable all warnings\n\
       \032    C/c enable/disable suspicious comment\n\
       \032    F/f enable/disable partially applied function\n\
       \032    M/m enable/disable overriden method\n\
       \032    P/p enable/disable partial match\n\
       \032    S/s enable/disable non-unit statement\n\
       \032    U/u enable/disable unused match case\n\
       \032    V/v enable/disable hidden instance variable\n\
       \032    X/x enable/disable all other warnings\n\
       \032    default setting is A (all warnings enabled)"),

     ("-dparsetree", Arg.Set dump_parsetree, " (undocumented)"),
     ("-drawlambda", Arg.Set dump_rawlambda, " (undocumented)"),
     ("-dlambda", Arg.Set dump_lambda, " (undocumented)"),
     ("-dinstr", Arg.Set dump_instr, " (undocumented)"),
     ("-ntyping", Arg.Set ntyping, " turn on the nML type-checker"),
     ("-nontyping", Arg.Clear ntyping, " turn off the nML type-checker") 
    ] file_argument usage;
  loop Format.std_formatter)

end

val _ = Printexc.catch Ntopmain.main ()
