(*********************************************************************)
(*   Hyun-Goo Kang, The LET Project, KAIST                           *)
(*-------------------------------------------------------------------*)
(*   Copyright(c) 2001 Research On Program Analysis System           *)
(*   National Creative Research Initiative Center                    *)
(*   Korea Advanced Institute of Science & Technology                *)
(*   http://ropas.kaist.ac.kr                                        *)
(*-------------------------------------------------------------------*)
(*  All rights reserved. This file is distributed under the terms of *)
(*  an Open Source License.                                          *)
(*********************************************************************)

structure Main =

struct
  exception DBError of string 
  exception Err of string
  exception SearchFail
  open Mysql_api     // use Mysql_api library

  fun open_database(db_name)  =
  let 	
    val null_mysql = mysql_null()
    val mysql = mysql_init(null_mysql)    
    val mysql1 = mysql_real_connect mysql "ropas.kaist.ac.kr" "guest" "1234" "" 3306 "" 0
    val iret = mysql_select_db mysql1 db_name
    val _ = if (iret = 0) then () else
    		(print_string "iret="; print_int(iret); print_newline();
    		 mysql_close(mysql1); raise DBError("couldn't select db"))
  in
    mysql1
  end  
  
  fun refined_open_db(db_name)  =
  let 	
    val null_mysql = mysql_null()
    val mysql = mysql_init(null_mysql)
    val mysql1 = mysql_refined_real_connect mysql "ropas.kaist.ac.kr" "guest" "1234" ""  // always use port 3306
    
    val iret = mysql_select_db mysql1 db_name
    val _ = if (iret = 0) then () else
    		(print_string "iret="; print_int(iret); print_newline();
    		 mysql_close(mysql1); raise DBError("couldn't select db"))
  in
    mysql1
  end  

  fun query_db(mysql) = let 
    val sSQL = "SELECT * FROM ropas_people" 
    val _ = (Format.printf "====  Qeurying Database ====\n";  
    	     Format.printf " Query string : select * from ropas_people\n";
    	     print_newline()) 
    val query_result = mysql_query mysql sSQL 
    val _ = if (query_result = 0) then ()  
    	    else  
    	      (Format.printf "Couldn't execute %s on the server !\n" sSQL;  
    	       print_newline(); 
    	       raise Err("query fail")) 
    val res = mysql_store_result(mysql) 
    
    // counting the number of rows and fields
    val num_of_row = mysql_num_rows res 
    val num_of_field = mysql_num_fields res 
    
    val _ = (Format.printf "num of row = %d" num_of_row; print_newline()) 
    val _ = (Format.printf "num of field = %d\n" num_of_field; print_newline()) 
    
    // get information for fields in a row   
    val field_list = ref [] 
    val fd = ref (mysql_fetch_field res) 
    val _ = while (is_mysql_field_null(!fd) <> true) do  
      	       (field_list := (!field_list)@[!fd]; 
       		fd := mysql_fetch_field(res) 
       		) 
       	    end 

    val row = ref (mysql_fetch_row res) 
    val counter = ref 0 

  in  
    // print each row	 
    for idx=0; idx<num_of_row; idx+1 do 
      let               
    	val _ = 1   
      in
      	// print current row
      	Format.printf "Record #%d:-\n" (counter++; !counter);      	
      	for k=0; k < num_of_field; (k+1) do
      	  let 
      	    val current_fld = List.nth !field_list k
      	    val current_fld_name = mysql_extract_field_name current_fld 
      	    val current_fld_value = mysql_extract_nth_row !row k
      	  in 
      	    (Format.printf "  Field #%d (%s): %s \n") (k+1) current_fld_name current_fld_value
	  end	      	    		  
      	end;
	      	
      	// move cursor to next row 
	row := mysql_fetch_row(res) 
      end
    end;

    Format.printf "\n\n====  Diagnostic info  ====\n";
    let val pszT = mysql_get_client_info() 
    in Format.printf "Client info: %s\n" pszT end;
  
    let val pszT = mysql_get_host_info(mysql) 
    in Format.printf "Host info: %s\n" pszT  end;
  
    let val pszT = mysql_get_server_info(mysql)
    in Format.printf "Server info: %s\n" pszT end
  end

  fun main() = 
   let  
      val x = 1     
      val my_data = refined_open_db("ropas")
      //val my_data = open_database("ropas")
      val _ = query_db(my_data) 
    in
      print_newline() 
    end  
  
  val _ = main()  

end 

