#include <windows.h>
#include <stdio.h>
#include <string.h>

#include "../../../../byterun/mlvalues.h"   // this is for c interfacing in nML
#include "../../../../byterun/alloc.h"

#include <mysql.h>     // this is for mySQL DB interfacing in C

#ifndef offsetof
#define offsetof(TYPE, MEMBER) ((size_t) &((TYPE *)0)->MEMBER)
#endif

/***********************************************/
/* this is for accessing abstract data in nML  */
/***********************************************/

value c_mysql_null(value unit) { 
	return (value)NULL;	
}	

value c_mysql_res_null(value unit) { 
	return (value)NULL;	
}	

value c_mysql_row_null(value unit) { 
	return (value)NULL;	
}	

value c_mysql_field_null(value unit) { 
	return (value)NULL;	
}	

value c_mysql_extract_field_name(value v_field) { 
	MYSQL_FIELD* field = (MYSQL_FIELD*)v_field;
	return (value)copy_string(field->name);
}	

value c_mysql_extract_field_table(value v_field) {
	MYSQL_FIELD* field = (MYSQL_FIELD*)v_field;
	return (value)copy_string(field->table);
}	

value c_mysql_extract_field_def(value v_field) { 
	MYSQL_FIELD* field = (MYSQL_FIELD*)v_field;
	return (value)copy_string(field->def);
}	
	
value c_mysql_extract_field_type(value v_field) {
	MYSQL_FIELD* field = (MYSQL_FIELD*)v_field;
	int ret;
	enum enum_field_types ty = field->type;
	switch (ty) {
	  case FIELD_TYPE_TINY: ret = 0;  break;
	  case FIELD_TYPE_SHORT: ret = 1; break;
	  case FIELD_TYPE_LONG: ret = 2;  break;
	  case FIELD_TYPE_INT24: ret = 3; break;
	  case FIELD_TYPE_LONGLONG : ret = 4 ;  break;
	  case FIELD_TYPE_DECIMAL : ret = 5;  break;
	  case FIELD_TYPE_FLOAT : ret = 6;  break;
	  case FIELD_TYPE_DOUBLE : ret = 7;  break;
	  case FIELD_TYPE_TIMESTAMP : ret = 8;  break;
	  case FIELD_TYPE_DATE : ret = 9;  break;
	  case FIELD_TYPE_TIME : ret = 10;  break;
	  case FIELD_TYPE_DATETIME : ret = 11;  break;
	  case FIELD_TYPE_YEAR : ret = 12;  break;
	  case FIELD_TYPE_STRING : ret = 13;  break;
	  case FIELD_TYPE_BLOB : ret = 14;  break;
	  case FIELD_TYPE_SET : ret = 15;  break;
	  case FIELD_TYPE_ENUM : ret = 16;  break;
  	  case FIELD_TYPE_NULL : ret = 17;  break;
	  //case FIELD_TYPE_CHAR : ret = 18;  break;
	  default : ret = 20; break;
	};
	return Val_int(ret);
}

value c_mysql_extract_nth_row(value v_row, value v_nth) {
	MYSQL_ROW row = (MYSQL_ROW)v_row;
	int nth = Int_val(v_nth);
	char buff[100];

	char* ret = row[nth];

	if (ret==NULL || !strlen(ret)) 
		return (value)copy_string("NULL");
	else
		return (value)copy_string(ret);
}

value c_is_mysql_null(value v_mysql) {
	MYSQL* mysql = (MYSQL*) v_mysql;
	if (mysql) 
		return Val_bool(0);
	else 
	    return Val_bool(1);
}	

value c_is_mysql_res_null(value v_res) {
	MYSQL_RES* res = (MYSQL_RES*) v_res;
	if (res) 
		return Val_bool(0);
	else 
	    return Val_bool(1);
}	

value c_is_mysql_row_null(value v_row) {
	MYSQL_ROW row = (MYSQL_ROW) v_row;
	if (row) 
		return Val_bool(0);
	else 
	    return Val_bool(1);
}	

value c_is_mysql_field_null(value v_field) { 
	MYSQL_FIELD* field = (MYSQL_FIELD*) v_field;

	if (field)
		return Val_bool(0);
	else 
	    return Val_bool(1);	
}


value c_mysql_get_option(value v_i) {
	int i = Int_val(v_i);
	
	switch(i) {
	 case 0 : return (value)MYSQL_OPT_CONNECT_TIMEOUT ;
	 case 1 : return (value)MYSQL_OPT_COMPRESS ;
	 case 2 : return (value)MYSQL_OPT_NAMED_PIPE ;
	 case 3 : return (value)MYSQL_INIT_COMMAND ;
	 case 4 : return (value)MYSQL_READ_DEFAULT_FILE ;
	 case 5 : return (value)MYSQL_READ_DEFAULT_GROUP ;
	 case 6 : return (value)MYSQL_SET_CHARSET_DIR ;
	 case 7 : return (value)MYSQL_SET_CHARSET_NAME ;
	 default : return (value)(-1);
	}	
}
	
/********************************************************
  nML to C MySQL API Interface functions implementation
*********************************************************/

value c_mysql_affected_rows(value v1) {
	MYSQL* mysql = (MYSQL*) v1;
	return (value)mysql_affected_rows(mysql);
}

value c_mysql_close(value v1) {
	MYSQL* mysql = (MYSQL*) v1;
	mysql_close(mysql);
	return Val_unit;
}	

value c_mysql_connect(value v1, value host, value user, value passwd) { 
	MYSQL* mysql = (MYSQL*) v1;
	const char* host1 = String_val(host);
	const char* user1 = String_val(user);
	const char* passwd1 = String_val(passwd);

	return (value)mysql_connect(mysql, host1, user1, passwd1);
}

value c_mysql_create_db(value v1, value v_db) {
	MYSQL* mysql = (MYSQL*) v1;
	const char* db = String_val(v_db);
	
	int ret = mysql_create_db(mysql,db);

	return Val_int(ret) ;
} 
	 
value c_mysql_data_seek(value v_res, value v_offset) {
	MYSQL_RES* res = (MYSQL_RES*) v_res;
	my_ulonglong offset = (my_ulonglong)Int_val(v_offset);
	mysql_data_seek(res, offset);
	return Val_unit;
}

value c_mysql_debug(value v1) { 
	char* debug = String_val(v1);
	return Val_unit;
}
 
value c_mysql_drop_db(value v_mysql, value v_db) { 
	MYSQL* mysql = (MYSQL*) v_mysql;
	const char* db = String_val(v_db);

	return Val_int(mysql_drop_db(mysql,db));
}

value c_mysql_dump_debug_info(value v_mysql) {
	MYSQL* mysql = (MYSQL*) v_mysql;
	
	return Val_int(mysql_dump_debug_info(mysql));
}
	 
value c_mysql_eof(value v_res) { 
	MYSQL_RES* res = (MYSQL_RES*) v_res;

	return Val_int((int)mysql_eof(res));
	// my_bool : zero if no error, or non-zero
}
 	 
value c_mysql_errno(value v_mysql) { 
	MYSQL* mysql = (MYSQL*) v_mysql;

	return Val_int((unsigned int)mysql_errno(mysql));
	// unsigned int : error code value, zero if no error
}
 
value c_mysql_error(value v_mysql) { 
	MYSQL* mysql = (MYSQL*) v_mysql;

	return (value)copy_string(mysql_error(mysql));
	// char* : error message for most recently invoked API fun,
	// "" : if no error
}

value c_mysql_escape_string(value v_to, value v_from, value v_from_length) {
	char* to = String_val(v_to);
	char* from = String_val(v_from);
	unsigned long from_length = (unsigned long)Int_val(v_from_length);
	
	unsigned long ret = mysql_escape_string(to,from,from_length);
	
	return Val_int((int)ret);
}	

value c_mysql_fetch_field(value v_res) { 
	MYSQL_RES* res = (MYSQL_RES*) v_res;

	MYSQL_FIELD* ret = mysql_fetch_field(res);

	return (value)ret;
	// the mysql_field structure for the current column,
	// null if no columns are left
}

value c_mysql_fetch_fields(value v_res) { 
	MYSQL_RES* res = (MYSQL_RES*) v_res;

	MYSQL_FIELD* ret = mysql_fetch_fields(res);

	return (value)ret;
	// an array of mysql_field structures for all columns of a result set	
}

value c_mysql_fetch_field_direct(value v_res, value v_fieldnr) {
	MYSQL_RES* res = (MYSQL_RES*) v_res;
	unsigned int fieldnr = (unsigned int)Int_val(v_fieldnr);

	MYSQL_FIELD* ret = mysql_fetch_field_direct(res, fieldnr);

	return (value)ret;
	// mysql_field structure for the specified column
}

value c_mysql_fetch_lengths(value v_res) {
	MYSQL_RES* res = (MYSQL_RES*) v_res;

	unsigned long* ret = mysql_fetch_lengths(res);

	return (value)ret;
	// unsigned long* : an array of unsigned long integers representing the size
	//	of each column, null of an error occurred
}
	 
value c_mysql_fetch_row(value v_res) { 
	MYSQL_RES* res = (MYSQL_RES*) v_res;

	MYSQL_ROW ret = mysql_fetch_row(res);

	return (value)ret;
	// a mysql_row structure for next row.
	// null if no more rows to retrieve or if an error occurred
}

value c_mysql_field_seek(value v_res, value v_offset) { 
	MYSQL_RES* res = (MYSQL_RES*) v_res;
	MYSQL_FIELD_OFFSET offset = (MYSQL_FIELD_OFFSET)v_offset;

	MYSQL_FIELD_OFFSET ret = mysql_field_seek(res,offset);

	return (value)ret;
	// previous value of the field cursor
}

value c_mysql_field_count(value v_mysql) { 
	MYSQL* mysql = (MYSQL*) v_mysql;

	unsigned int ret = mysql_field_count(mysql);

	return Val_int((int)ret);
	// unsigned int : the number of fields in a result set
}

value c_mysql_field_tell(value v_res) { 
	MYSQL_RES* res = (MYSQL_RES*) v_res;

	MYSQL_FIELD_OFFSET ret = mysql_field_tell(res);

	return (value)ret;
	// position of field cursor used for the last mysql_fetch_field
	// can be used as an argument to mysql_field_seek
}
	    
value c_mysql_free_result(value v_res) {
	MYSQL_RES* res = (MYSQL_RES*) v_res;

	mysql_free_result(res);

	return Val_unit;
}

value c_mysql_get_client_info(value unit) { 
	char* ret = mysql_get_client_info();

	return (value)copy_string(ret);
	// the mysql client library version
}

value c_mysql_get_host_info(value v_mysql) {
	MYSQL* mysql = (MYSQL*)v_mysql;

	char* ret = mysql_get_host_info(mysql);

	return (value)copy_string(ret);
	// char* : the server host name and the connection type
}

value c_mysql_get_proto_info(value v_mysql) {
	MYSQL* mysql = (MYSQL*)v_mysql;

	unsigned int ret = mysql_get_proto_info(mysql);

	return Val_int((int)ret);
	// unsigned int : protocol version used by the current connection
}

value c_mysql_get_server_info(value v_mysql) {
	MYSQL* mysql = (MYSQL*)v_mysql;

	char* ret = mysql_get_server_info(mysql);

	return (value)copy_string(ret);	
	// char* : the server version number
}

value c_mysql_info(value v_mysql) { 
	MYSQL* mysql = (MYSQL*)v_mysql;

	char* ret = mysql_info(mysql);

	return (value)copy_string(ret);	
	// char* : additional information about the most recently executed query.  
	// 	null if no information is available for the query 
}


// allocates or initializes a MYSQL object suitable for mysql_real_connect().
// if mysql is a NULL pointer, the function allocates, initializes and returns a new object.
// otherwise the object is initialized and the address of the object is returned.
// if mysql_init() allocates a new object, it will be freed when mysql_close() is 
// called to close the connection.	 		
//value c_mysql_init_org(value v_mysql) {  
//	MYSQL* mysql = (MYSQL*)v_mysql;
//
//	MYSQL* ret = mysql_init(mysql);
//
//	return (value)ret;
//}

value c_mysql_init(value v_mysql) {  
	MYSQL* mysql = (MYSQL*)v_mysql; 

	if ( (mysql = mysql_init((MYSQL*) mysql) )) { 
		MYSQL* ret = (MYSQL*)mysql_init(mysql);
		return (value)ret;
	}
	else {
		printf( "Can't connect to the mysql server !\n");
		mysql_close( mysql );
		return (value)NULL; 
	}
}


value c_mysql_insert_id (value v_mysql) {  
	MYSQL* mysql = (MYSQL*)v_mysql;

	my_ulonglong ret = mysql_insert_id(mysql);
	// the value of the AUTO_INCREMENT field that was updated by the previous query.
    // returns zero if there was no previous query on the connection 
	// or if the query didn't update an AUTO_INCREMENT value

	return Val_int((int)ret);
}

// asks the server to kill the thread specified by pid
value c_mysql_kill(value v_mysql, value v_pid) { 
	MYSQL* mysql = (MYSQL*)v_mysql;
	unsigned long pid = (unsigned long)Int_val(v_pid);

	int ret = mysql_kill(mysql,pid);

	return Val_int(ret);
	// int : zero for success, or non-zero
}

value c_mysql_list_dbs(value v_mysql, value v_wild) { 
	MYSQL* mysql = (MYSQL*)v_mysql;
	char* wild = String_val(v_wild);
	MYSQL_RES* ret = NULL;

	int null_str_wild = strcmp(wild,"");
	if (!null_str_wild) wild = NULL;

	ret = mysql_list_dbs(mysql, wild);

	return (value)ret;
}

value c_mysql_list_fields(value v_mysql, value v_table, value v_wild) { 
	MYSQL* mysql = (MYSQL*)v_mysql;
	const char* table = String_val(v_table);
	const char* wild = String_val(v_wild);
	MYSQL_RES* ret = NULL;
	
	int null_str_wild = strcmp(wild,"");
	if (!null_str_wild) wild = NULL;

	ret = mysql_list_fields(mysql, table, wild);

	return (value)ret;
}
	 
value c_mysql_list_processes(value v_mysql) {
	MYSQL* mysql = (MYSQL*)v_mysql;	
	
	MYSQL_RES* ret = mysql_list_processes(mysql);

	return (value)ret;
}	
	 
value c_mysql_list_tables(value v_mysql, value v_wild) { 
	MYSQL* mysql = (MYSQL*)v_mysql;	
	const char* wild = String_val(v_wild);
	MYSQL_RES* ret = NULL;

	int null_str_wild = strcmp(wild,"");
	if (!null_str_wild) wild = NULL;	

	ret = mysql_list_tables(mysql, wild);

	return (value)ret;
}

value c_mysql_num_fields(value v_res) {
	MYSQL_RES* res = (MYSQL_RES*)v_res;

	unsigned int ret = mysql_num_fields(res);

	return Val_int((int)ret);
}

value c_mysql_num_rows(value v_res) { 
	MYSQL_RES* res = (MYSQL_RES*)v_res;

	my_ulonglong ret = mysql_num_rows(res); 
	
	return Val_int((int)ret);
}

value c_mysql_options(value v_mysql, value v_option, value args) { 
	MYSQL* mysql = (MYSQL*) v_mysql;
	enum mysql_option option = (enum mysql_option)(v_option);//Int_val(v_option);
	const char* arg = String_val(args);

	int ret = mysql_options(mysql,option,arg);

	return Val_int(ret);
	// int : zero fo success, or non-zero
}

value c_mysql_ping(value v_mysql) { 
	MYSQL* mysql = (MYSQL*) v_mysql;

	int ret = mysql_ping(mysql);

	return Val_int(ret);	
	// int : zero if the server is alive, or non-zero
}
	 
value c_mysql_query(value v_mysql, value v_query) { 
	MYSQL* mysql = (MYSQL*) v_mysql;
	char* query = String_val(v_query);

	int ret = mysql_query(mysql, query);

	return Val_int(ret);
	// int : zero if the query was successful		
}

//value c_mysql_real_connect(value v_mysql, value v_host, value v_user, value v_passwd,
//			   value v_db, value v_port, value v_unix_socket, value v_client_flag)	{
//	MYSQL* mysql = (MYSQL*) v_mysql;
//	const char* host = String_val(v_host);
//	const char* user = String_val(v_user);
//	const char* passwd = String_val(v_passwd);
//	const char* db = String_val(v_db);	
//	unsigned int port = (unsigned int)Int_val(v_port);
//	const char* unix_socket = String_val(v_unix_socket);
//	unsigned int client_flag = (unsigned int)Int_val(v_client_flag);
//
//	MYSQL* mysql1 = mysql_init((MYSQL*)0);
//	MYSQL* ret = mysql_real_connect(mysql1, NULL, NULL, NULL, NULL, port, NULL, 0);
//
//	return (value)ret;  
//}

value c_mysql_real_connect_native(value v_mysql, value v_host, value v_user, value v_passwd,
			   value v_db, value v_port, value v_unix_socket, value v_client_flag)	{
	MYSQL* mysql = (MYSQL*) v_mysql;
	const char* host = String_val(v_host);
	const char* user = String_val(v_user);
	const char* passwd = String_val(v_passwd);
	const char* db = String_val(v_db);	
	unsigned int port = (unsigned int)Int_val(v_port);
	const char* unix_socket = String_val(v_unix_socket);
	unsigned int client_flag = (unsigned int)Int_val(v_client_flag);
	MYSQL* ret;

	int null_str_host = strcmp(host,"");
	int null_str_user = strcmp(user,"");
	int null_str_passwd = strcmp(passwd,"");
	int null_str_db = strcmp(db,"");
	int null_str_unix_socket = strcmp(unix_socket,"");
	
	if (!null_str_host) host = NULL;
	if (!null_str_user) user = NULL;
	if (!null_str_passwd) passwd = NULL;	
	if (!null_str_db) db = NULL;
	if (!null_str_unix_socket) unix_socket = NULL;
	
	ret = mysql_real_connect(mysql, host, user, passwd, db, port, unix_socket, client_flag);

	return (value)ret;  
}

value c_mysql_real_connect_bytecode(value* argv, int argn) { 
	return c_mysql_real_connect_native(argv[0], argv[1], argv[2], argv[3],
			     argv[4], argv[5], argv[6], argv[7]);
}	
	 
value c_mysql_real_query(value v_mysql, value v_query, value v_length) { 
	MYSQL* mysql = (MYSQL*) v_mysql;
	const char* query = String_val(v_query);
	unsigned int length = (unsigned int)Int_val(v_length);

	int ret = mysql_real_query(mysql, query, length);

	return Val_int(ret);  // zero, if success
}

value c_mysql_reload(value v_mysql) { 
	MYSQL* mysql = (MYSQL*) v_mysql;

	int ret = mysql_reload(mysql);

	return Val_int(ret); // int : zero for success
}
	 
value c_mysql_row_seek(value v_res, value v_offset) {
	MYSQL_RES* res = (MYSQL_RES*) v_res;
	MYSQL_ROW_OFFSET offset = (MYSQL_ROW_OFFSET)v_offset;

	MYSQL_ROW_OFFSET ret = mysql_row_seek(res,offset);

	return (value)ret; // int : zero for success
}
	 
value c_mysql_row_tell(value v_res) { 
	MYSQL_RES* res = (MYSQL_RES*) v_res;
	//MYSQL_ROW_OFFSET offset = (MYSQL_ROW_OFFSET)v_offset;

	MYSQL_ROW_OFFSET ret = mysql_row_tell(res);

	return (value) ret; 
}

value c_mysql_select_db(value v_mysql, value v_db) { 
	MYSQL* mysql = (MYSQL*) v_mysql;
	const char* db = String_val(v_db);

	int ret = mysql_select_db(mysql, db);
	
	return Val_int(ret);
	// int : zero for success
}

value c_mysql_shutdown(value v_mysql) {
	MYSQL* mysql = (MYSQL*) v_mysql;
	
	int ret = mysql_shutdown(mysql);

	return Val_int(ret); 
	// int : zero for success	
}	

value c_mysql_stat(value v_mysql) { 
	MYSQL* mysql = (MYSQL*) v_mysql;

	char* ret = mysql_stat(mysql);

	return (value)copy_string(ret);
	// char*  : a character string describing the server status
	//  	null if an error occurred
}
	
value c_mysql_store_result(value v_mysql) { 
	MYSQL* mysql = (MYSQL*) v_mysql;

	MYSQL_RES* ret = mysql_store_result(mysql); 

	return (value)ret;	
}
	 	 
value c_mysql_thread_id(value v_mysql) { 
	MYSQL* mysql = (MYSQL*) v_mysql;

	unsigned long ret = mysql_thread_id(mysql);

	return Val_int((int)ret);
}
	 

value c_mysql_use_result(value v_mysql) { 
	MYSQL* mysql = (MYSQL*) v_mysql;

	MYSQL_RES* ret = mysql_use_result(mysql);

	return (value) ret;
}

//value c_mysql_change_user(value v1, value user, value passwd, value db) { 
//	MYSQL* mysql = (MYSQL*) v1;
//	const char* user1 = String_val(user);
//	const char* passwd1 = String_val(passwd);
//	const char* db1 = String_val(db);
//	//strange.. following function("mysql_change_user") doesn't work
//	//return Val_int(mysql_change_user(mysql, user1, passwd1, db1));
//}
 	 
//value c_mysql_character_set_name(value v1) { 
//	MYSQL* mysql = (MYSQL*) v1;
//	const char* ret = mysql_character_set_name(mysql);
//	return (value)copy_string((char*)ret);
//}

//value c_mysql_real_escape_string(value v_mysql, value v_to, value v_from, value v_length) {
//	MYSQL* mysql = (MYSQL*) v_mysql;
//	char* to = String_val(v_to);
//	const char* from = String_val(v_from);
//	unsigned int length = Int_val(v_length);
//	
//	unsigned int ret = mysql_real_escape_string(mysql, to, from, length);
//
//	return Val_int(ret);
//}

//value c_test_native(value v1, value v2, value v3, value v4, value v5, value v6, value v7) {
//	const char* test_str = String_val(v1);
//		
//	printf("test_string = %s\n",test_str);
//	
//	return Val_int(18);
//}	

//value c_test_bytecode(value *argv, int argn) { 
//
//	return c_test_native(argv[0], argv[1], argv[2], argv[3],
//			     argv[4], argv[5], argv[6]);
//}	

value c_mysql_refined_real_connect(value v_mysql, value v_host, 
				   value v_user, value v_passwd, value v_db) {
	MYSQL* mysql = (MYSQL*) v_mysql;
	const char* host = String_val(v_host);
	const char* user = String_val(v_user);
	const char* passwd = String_val(v_passwd);
	const char* db = String_val(v_db);	

	MYSQL* ret;

	int null_str_host = strcmp(host,"");
	int null_str_user = strcmp(user,"");
	int null_str_passwd = strcmp(passwd,"");
	int null_str_db = strcmp(db,"");
		
	if (!null_str_host) host = NULL;
	if (!null_str_user) user = NULL;
	if (!null_str_passwd) passwd = NULL;	
	if (!null_str_db) db = NULL;
		
	ret = mysql_real_connect(mysql, host, user, passwd, db, MYSQL_PORT, NULL, 0);

	return (value)ret;  
}

/*
int  main() {
  int x = 0;
  
  printf ("this is just test!\n");
  return 0;
}
*/
