type mysql     		(* MYSQL*               *)
type mysql_res 		(* MYSQL_RES*           *)
type mysql_row		(* MYSQL_ROW            *)
type mysql_field	(* MYSQL_FIELD*         *)
type mysql_field_offset (* MYSQL_FIELD_OFFSET   *)
type mysql_row_offset   (* MYSQL_ROW*           *)   
type my_ulonglong = int	(* unsigned long long   *)
type mysql_option       (* enum mysql_option    *)
type my_bool = int	(* char			*)
type lengths		(* unsigned long*       *)


(**********************************************)
(* this is for accessing abstract data in nML *)
(**********************************************) 

external mysql_null : unit -> mysql = "c_mysql_null"
external mysql_res_null : unit -> mysql_res = "c_mysql_res_null"
external mysql_row_null : unit -> mysql_row = "c_mysql_row_null"
external mysql_field_null : unit -> mysql_field = "c_mysql_field_null"

external is_mysql_null : mysql -> bool = "c_is_mysql_null"
external is_mysql_res_null : mysql_res -> bool = "c_is_mysql_res_null"
external is_mysql_row_null : mysql_row -> bool = "c_is_mysql_row_null"
external is_mysql_field_null : mysql_field -> bool = "c_is_mysql_field_null"

external mysql_extract_field_name : mysql_field -> string = "c_mysql_extract_field_name"
external mysql_extract_field_table : mysql_field -> string = "c_mysql_extract_field_table"
external mysql_extract_field_def : mysql_field -> string = "c_mysql_extract_field_def"
external mysql_extract_field_type : 
	mysql_field 
	-> int 	(* 0 : FIELD_TYPE_TINY *)
		(* 1 : FIELD_TYPE_SHORT *)
		(* 2 : FIELD_TYPE_LONG *)
		(* 3 : FIELD_TYPE_INT24 *)
		(* 4 : FIELD_TYPE_LONGLONG *)
		(* 5 : FIELD_TYPE_DECIMAL *)
		(* 6 : FIELD_TYPE_FLOAT *)
		(* 7 : FIELD_TYPE_DOUBLE *)
		(* 8 : FIELD_TYPE_TIMESTAMP *)
		(* 9 : FIELD_TYPE_DATE *)
		(* 10 : FIELD_TYPE_TIME *)
		(* 11 : FIELD_TYPE_DATETIME *)
		(* 12 : FIELD_TYPE_YEAR *)
		(* 13 : FIELD_TYPE_STRING *)
		(* 14 : FIELD_TYPE_BLOB *)
		(* 15 : FIELD_TYPE_SET *)
		(* 16 : FIELD_TYPE_ENUM *)
		(* 17 : FIELD_TYPE_NULL *)
		(* 18 : FIELD_TYPE_CHAR *)
 = "c_mysql_extract_field_type"

external mysql_extract_nth_row : 
	mysql_row -> int -> string
 = "c_mysql_extract_nth_row"


external mysql_get_option :
	 int 
	 -> mysql_option 
	 (* mysql_option_get 0 = MYSQL_OPT_CONNECT_TIMEOUT *)
	 (* 1 : MYSQL_OPT_COMPRESS *)
	 (* 2 : MYSQL_OPT_NAMED_PIPE *)
	 (* 3 : MYSQL_INIT_COMMAND *)
	 (* 4 : MYSQL_READ_DEFAULT_FILE *)
	 (* 5 : MYSQL_READ_DEFAULT_GROUP *)
	 (* 6 : MYSQL_SET_CHARSET_DIR *)
	 (* 7 : MYSQL_SET_CHARSET_NAME *)
 = "c_mysql_get_option"	 
  			
  			
  			
(****************************************************)
(*   default mysql api interface                    *)
(****************************************************) 				

external mysql_affected_rows : mysql -> my_ulonglong 
 = "c_mysql_affected_rows"

external mysql_close : mysql -> unit = "c_mysql_close"

external mysql_connect : 
 	 mysql 		(* mysql *)
 	 -> string      (* const char* host  *)
	 -> string      (* const char* user *)
	 -> string      (* const char* passwd *)
	 -> mysql    
 = "c_mysql_connect"
 
external mysql_create_db : 
	 mysql 		(* mysql *)
	 -> string	(* const char* db *)
	 -> int		(* int : zero if success or non-zero *)
 = "c_mysql_create_db"	 
	 
external mysql_data_seek :
	 mysql_res	(* result *)
	 -> my_ulonglong  (* unsigned long long_offset *)
	 -> unit	(* unit *)
 = "c_mysql_data_seek"	 
	 
external mysql_debug :
	 string		(* char* debug *)
	 -> unit	(* unit	 *)
 = "c_mysql_debug"	 	 
 
external mysql_drop_db :
	 mysql		(* mysql *)
	 -> string 	(* const char* db *)
	 -> int		(* int : zero if success, or non-zero *)
 = "c_mysql_drop_db" 

(*
 instruct the server to write somedebug information to the log
 the connected user must have the process privilege for this to work
*)
external mysql_dump_debug_info :
	 mysql		(* mysql *)
	 -> int 	(* int : zero if success, or non-zero*)
 = "c_mysql_dump_debug_info"	 
	 
external mysql_eof :
	 mysql_res	(* result*)
	 -> my_bool	(* my_bool : zero if no error, or non-zero*)
 = "c_mysql_eof"
 	 
external mysql_errno : 
	 mysql		(* mysql*)
	 -> int		(* unsigned int : error code value, zero if no error*)
 = "c_mysql_errno"	 
	 
external mysql_error :
	 mysql		(* mysql*)
	 -> string	(* char* : error message for most recently invoked API fun,*)
	 		(* "" if no error*)
 = "c_mysql_error"	 		

external mysql_escape_string :
	string 		(* char* to *)
        -> string 	(* char* from *)
        -> int		(* unsigned long from_length *)
        -> int 		(* unsigned long *)
 = "c_mysql_escape_string"        

external mysql_fetch_field :
	 mysql_res	(* result*)
	 -> mysql_field (* the mysql_field structure for the current column,*)
	 		(* null if no columns are left*)
 = "c_mysql_fetch_field"	 		

external mysql_fetch_fields :
	 mysql_res	(* result *)
	 -> mysql_field (* an array of mysql_field structures for all columns of a result set *)
 = "c_mysql_fetch_fields"	 
	 
external mysql_fetch_field_direct :
	 mysql_res	(* result *)
	 -> int		(* unsigned int fieldnr *)
	 -> mysql_field (* mysql_field structure for the specified column *)
 = "c_mysql_fetch_fields_direct"	 

external mysql_fetch_lengths :
	 mysql_res 	(* result *)
	 -> lengths	(* unsigned long* : an array of unsigned long integers representing the size*)
	 		(*	of each column, null of an error occurred *)
 = "c_mysql_fetch_lenghths"	 		
	 
external mysql_fetch_row :
	 mysql_res	(* result *)
	 -> mysql_row	(* a mysql_row structure for next row.*)
	 		(* null if no more rows to retrieve or if an error occurred*)
 = "c_mysql_fetch_row"	 		

external mysql_field_seek :
	 mysql_res	(* result*)
	 -> mysql_field_offset (* offset*)
	 -> mysql_field_offset (* previous value of the field cursor*)
 = "c_mysql_field_seek"	 

external mysql_field_count :
	 mysql		(* mysql *)
	 -> int	 	(* unsigned int : the number of fields in a result set *)
 = "c_mysql_field_count"	 

external mysql_field_tell :
	 mysql_res	(* result *)
	 -> mysql_field_offset	(* = unsigned int *)
	    (* position of field cursor used for the last mysql_fetch_field *)
	    (* can be used as an argument to mysql_field_seek *)
 = "c_mysql_field_tell"	    
	    
external mysql_free_result :
	 mysql_res	(* result *)
	 -> unit	(* () *)
 = "c_mysql_free_result"	 

external mysql_get_client_info :
	 unit		(* unit *)
	 -> string	(* char* : the mysql client library version *)
 = "c_mysql_get_client_info"	 

external mysql_get_host_info :
	 mysql		(* mysql *)
	 -> string	(* char* : the server host name and the connection type *)
 = "c_mysql_get_host_info"	 

external mysql_get_proto_info :
	 mysql 		(* mysql *)
	 -> int		(* unsigned int : protocol version used by the current connection *)
 = "c_mysql_get_proto_info"

external mysql_get_server_info : 
	 mysql		(* mysql *)
	 -> string	(* char* : the server version number *)
 = "c_mysql_get_server_info"	 

external mysql_info :
	 mysql		(* mysql *)
	 -> string	(* char* : additional information about the most recently executed query. *) 
	 		(* 	null if no information is available for the query *)
 = "c_mysql_info"	 		

(*	 
 allocates or initializes a MYSQL object suitable for mysql_real_connect().
 if mysql is a NULL pointer, the function allocates, initializes and returns a new object.
 otherwise the object is initialized and the address of the object is returned.
 if mysql_init() allocates a new object, it will be freed when mysql_close() is 
 called to close the connection.	 		
*)

external mysql_init :
	 mysql		(* mysql : usually null. get this from c function*)
	 -> mysql	(* mysql *)
 = "c_mysql_init"	 
 
(*
external mysql_init_org : 
	 mysql		(* mysql : usually null. get this from c function*)
	 -> mysql	(* mysql *)
 = "c_mysql_init_org"	 
*)

external mysql_insert_id :
	 mysql		(* mysql*)
	 -> my_ulonglong  (* the value of the AUTO_INCREMENT field that was updated by the previous query.*)
	 		  (* returns zero if there was no previous query on the connection *)
	 		  (* or if the query didn't update an AUTO_INCREMENT value*)
 = "c_mysql_insert_id"	 		  
 
(* asks the server to kill the thread specified by pid*)
external mysql_kill : 
	 mysql		(* mysql*)
	 -> int		(* unsigned long pid *)
	 -> int		(* int : zero for success, or non-zero*)
 = "c_mysql_kill"

external mysql_list_dbs :
	 mysql
	 -> string	(* const char* wild*)
	 -> mysql_res	
 = "c_mysql_list_dbs"	 

external mysql_list_fields :
	 mysql
	 -> string	(* const char* table*)
	 -> string	(* const char* wild*)
	 -> mysql_res	 
 = "c_mysql_list_fields"	 
	 
external mysql_list_processes :
	 mysql
	 -> mysql_res
 = "c_mysql_list_processes"	 
	 
external mysql_list_tables :
	 mysql
	 -> string	(* const char* wild *)
	 -> mysql_res
 = "c_mysql_list_tables"	 

external mysql_num_fields : 
	 mysql_res 
	 -> int		(* unsigned int*)
 = "c_mysql_num_fields"	 

external mysql_num_rows :
	 mysql_res
	 -> my_ulonglong 
 = "c_mysql_num_rows"	 

external mysql_options :
	 mysql
	 -> mysql_option  (* obtained by mysql_get_option function *)
	 -> string 	(* const char* arg  *)
	 -> int 	(* int : zero fo success, or non-zero *)
 = "c_mysql_options"	 

external mysql_ping : 
	 mysql
	 -> int		(* int : zero if the server is alive, or non-zero *)
 = "c_mysql_ping"	 
	 
external mysql_query :
	 mysql
	 -> string 	(* const char* query *)
	 -> int		(* int : zero if the query was successful,  *)
 = "c_mysql_query"

external mysql_real_connect : (* if MYSQL_VERSION_ID >= 32200 *)
	 mysql
	 -> string	(* const char* host  *)
	 -> string	(* const char* user *)
	 -> string	(* const char* passwd *)
	 -> string 	(* const char* db *)
	 -> int 	(* unsigned int port *)
	 -> string	(* const char* unix_socket *)
	 -> int		(* unsigned int client_flag *)
	 -> mysql
 = "c_mysql_real_connect_bytecode" "c_mysql_real_connect_native"

external mysql_real_query : 
	 mysql
	 -> string	(* const char* query *)
	 -> int		(* unsigned int length *)
	 -> int		(* int : zero if success *)
 = "c_mysql_real_query"

external mysql_reload :
	 mysql
	 -> int		(* int : zero for success *)
 = "c_mysql_reload"
	 
external mysql_row_seek :
	 mysql_res
	 -> mysql_row_offset
	 -> mysql_row_offset 		(* int : zero for success  *)
 = "c_mysql_row_seek"

external mysql_row_tell :
	 mysql_res
	 -> mysql_row_offset   (* = MYSQL_ROW* *)
 = "c_mysql_row_tell"

external mysql_select_db :
	 mysql
	 -> string	(* const char* db  *)
	 -> int		(* int : zero for success *)
 = "c_mysql_select_db"

external mysql_shutdown :
	 mysql
	 -> int		(* int : zero for success *)
 = "c_mysql_shutdown"

external mysql_stat :
 	 mysql
 	 -> string	(* char*  : a character string describing the server status *)
 	 		(*  	null if an error occurred *)
 = "c_mysql_stat"

external mysql_store_result :
	 mysql
	 -> mysql_res	
 = "c_mysql_store_result"
	 	 
external mysql_thread_id :
	 mysql
	 -> int		(* unsigned long *)
 = "c_mysql_thread_id"	 

external mysql_use_result :
	 mysql
	 -> mysql_res
 = "c_mysql_use_result"


(*************************************************)
(*	removed library due to no definitionr  	 *)
(*-----------------------------------------------*)
(* it's likely that this doesn't exist		 *)
(* even thought it's spec is in the table of 	 *)
(*                   function in manual		 *)
(*************************************************)

(*
external mysql_slave :
external mysql_stop_slave :	 
external mysql_thread_save :
*)


(*************************************************)
(*	refined function definitions by kang  	 *)
(*-----------------------------------------------*)
(*						 *)
(*************************************************)

(* we can use naive style till 5 argument function *)
external mysql_refined_real_connect : 
	 mysql
	 -> string	(* const char* host  *)
	 -> string	(* const char* user *)
	 -> string	(* const char* passwd *)
	 -> string 	(* const char* db *)
	 -> mysql
 = "c_mysql_refined_real_connect" 


(*************************************************)
(*	   removed library due to link error  	 *)
(*-----------------------------------------------*)
(* it's likely that 				 *)
(*       libmysql.lib don't have this functions, *)
(* even thought it's spec is in mysql.h		 *)
(*************************************************)

(*
external mysql_change_user : 
 	 mysql  	(* mysql *)
 	 -> string 	(* const char* user *)
	 -> string 	(* const char* passwd *)
	 -> string 	(* const char* db *)
	 -> my_bool 	(* zero for success or non-zero *)
 = "c_mysql_change_user"
 	 
external mysql_character_set_name :
	 mysql      	(* mysql *)
	 -> string      (* const char* default character set *)
 = "c_mysql_character_set_name"

external mysql_real_escape_string :
	 mysql
	 -> string	(* char* to *)
	 -> string	(* const char* from*)
	 -> int		(* unsigned int length*)
	 -> int		(* unsigned int *)
 = "c_mysql_real_escape_string"	 
*)

(*
external test : string -> string -> string -> string -> string -> int -> string -> int
 = "c_test_bytecode" "c_test_native"
*) 

 
